#! /bin/sh

#
# @(#) Module de fonctions specifiques HPU
# 

#
#
# Liste des fonctions 
# grep '^[^ ]* ()' init_hpu_inst.sh | sort
#
#

### DEBUG ###
# set -x
set -u
### FIN DEBUG ###



###########################################################################
# hpuRetrieveServiceSettings
###########################################################################

#
# Recuperation des valeurs des variables
# HPU_SERVICE_NAME
# HPU_SERVICE_PORT
# du fichier de parametres d'installation
#
# Les variables sont initialisees dans une action AddServicesINST () mais 
# elles ne sont pas visibles dans les autres actions.
#
#
# Aucun argument d'appel.
# 
# Specifique HPU.
#
# 
# 
# 

hpuRetrieveServiceSettings ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> hpuRetrieveServiceSettings"
    DisplayTrace ""
    DisplayTrace ""


    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace "    Retrieval of parameter \"HPU_SERVICE_NAME\" from installation settings file"
    
        HPU_SERVICE_NAME=`GetSettingsVariable "HPU_SERVICE_NAME"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"HPU_SERVICE_NAME\" not found in ${FILE_INSTALL_SETTINGS}"
            RETURN_CODE=1
        else    
            DisplayTrace "    HPU_SERVICE_NAME  : \"${HPU_SERVICE_NAME}\" "
        fi
    fi


    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace "    Retrieval of parameter \"HPU_SERVICE_PORT\" from installation settings file"
        HPU_SERVICE_PORT=`GetSettingsVariable "HPU_SERVICE_PORT"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"HPU_SERVICE_PORT\" not found in ${FILE_INSTALL_SETTINGS}"
            RETURN_CODE=1
        else    
            DisplayTrace "    HPU_SERVICE_PORT  : \"${HPU_SERVICE_PORT}\" "
        fi
    fi
    
    return ${RETURN_CODE}
}


##########################################################################
# hpuSearchICULibrary
###########################################################################

# Recherche des bibliotheques ICU dans l'installation HPU existante 
# pour memorisation eventuelle de leur chemin (cas du FixPak)
# 
# Les chemins sont memorises dans le fichier de parametres d'installation 
# (nom contenu dans la variable globale FILE_INSTALL_SETTINGS.)
#
#
# Ce traitement permet d'initialiser les chemins dans le fichier de parametres
# lors de l'installation d'un FP dans le cas ou version presente de HPU (Full/T&B/WGE)
# n'a pas ces chemins memorises (HPU 3.1.x anterieurs au FP 3.1.2.
#
# Argument 1 : nom section  (il doit etre le meme dans le fichier FULL/WGE/T&B et FP)
#              La section doit etre associee a l'action d'installation de la 
#              bibliotheque 
# Argument 2 : nom du fichier bibliotheque ICU
#              Ce nom doit etre present dans la section indiquee en premier argument
# Argument 3 : nom de l'identificateur memorisant le chemin d'acces a la
#              bibliotheque ICU
#
#
# Fonction appelee dans le cadre d'une installation FixPak.
#
#
# Cette fonction est tres specifique a HPU.
#
### Suppression fichiers temporaires !!! ###
### Tester cas d'erreur                  ###
#
#
#

hpuSearchICULibrary ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> hpuSearchICULibrary"
    DisplayTrace ""
    DisplayTrace ""


    if [ $# -ne 3 ]
    then
        DisplayError "Missing arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        NAME_ICU_SECTION=$1
        NAME_ICU_LIBRARY_FILE=$2
        NAME_ICU_PATH_IDENTIFIER=$3
        
        
        DisplayTrace "    NAME_ICU_SECTION         : \"${NAME_ICU_SECTION}\" "
        DisplayTrace "    NAME_ICU_LIBRARY_FILE    : \"${NAME_ICU_LIBRARY_FILE}\" "
        DisplayTrace "    NAME_ICU_PATH_IDENTIFIER : \"${NAME_ICU_PATH_IDENTIFIER}\" "
        
        # Recherche si l'identificateur figure bien dans le fichier de 
        # parametres d'installation

        ${GREP} "${NAME_ICU_PATH_IDENTIFIER}" "${FILE_INSTALL_SETTINGS}" >/dev/null 2>&1
        if [ $? -ne 0 ]
        then
            DisplayTrace "    Settings parameter \"${NAME_ICU_PATH_IDENTIFIER}\" missing"

            # Si parametre d'installation non trouve : reconstitution et memorisation 
            # du parametre
        
            # Recherche de la section dans le fichier .INI de l'installation 
            # existante Full/WGE/T&B

            FILE_PLATFORM_CONFIG_EXISTING="install_${PLATFORM}.ini"
            # Fichier temporaire contenant la section existante
            FILE_SECTION_EXISTING="tmp_section_icu"

            DisplayTrace "    FILE_PLATFORM_CONFIG_EXISTING : \"${FILE_PLATFORM_CONFIG_EXISTING}\" "

            CheckReadableFile "${LOCATION_PATH}/install/${FILE_PLATFORM_CONFIG_EXISTING}"
            

            # Controle presence section
            ${GREP} '^\['"${NAME_ICU_SECTION}"'\]' "${LOCATION_PATH}/install/${FILE_PLATFORM_CONFIG_EXISTING}" >/dev/null 2>&1
            if [ $? -eq 0 ]
            then
                # Extraction de la section dans un fichier separe pour traitement
                ${SED} -e '/./{H;$!d;}' -e 'x;/\['"${NAME_ICU_SECTION}"'\]/!d;' \
                    "${LOCATION_PATH}/install/${FILE_PLATFORM_CONFIG_EXISTING}" | ${SED} -e '/^$/d' \
                    > "${FILE_SECTION_EXISTING}"
                if [ $? -eq 0 ]
                then
                    # Extraction parametre "search_directories" (optionnel en general
                    # mais doit etre present ici) 

                    SEARCH_DIRECTORIES=`GetINIValue "${FILE_SECTION_EXISTING}" "search_directories"`
                    RETURN_CODE=$?

                    if [ "${RETURN_CODE}" -eq 0 ]
                    then
                        DisplayTrace "    \"search_directories\" parameter present" 
                        DisplayTrace "    SEARCH_DIRECTORIES : ${SEARCH_DIRECTORIES}"
                    else    
                        if [ "${RETURN_CODE}" -eq 2 ]
                        then
                            # Parametre absent
                            # Le parametre est facultatif mais il doit etre present 
                            # dans le cas present
                            DisplayTrace "    No \"search_directories\" parameter present"
                            SEARCH_DIRECTORIES=""
                            RETURN_CODE=1
                        else    
                            # Cas improbable
                            RETURN_CODE=1
                            return ${RETURN_CODE}
                        fi    
                    fi
                fi        
            else    
                DisplayTraceAndError "No \"${NAME_ICU_SECTION}\" section found"
            fi    
        
        
            # Traitement  des fichiers concernes par la section dans le fichier .INI
            # de l'installation existante Full/WGE/T&B
            # - Recherche du fichier dans les repertoires concernes (valeur de
            #   search_directories)
            # - Memorisation (dans le fichier de parametres d'installation) du 
            #   chemin trouve avec la variable associee au fichier
            
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                # Extraction du parametre "file" pour la bibliotheque recherchee
                LINE_ICU_FILE=`${GREP} '^file='"${NAME_ICU_LIBRARY_FILE}:" "${FILE_SECTION_EXISTING}" 2>/dev/null`
                if [ $? -eq 0 ]
                then
                    DisplayTrace "    LINE_ICU_FILE : \"${LINE_ICU_FILE}\"" 

                    IDENTIFIER=`echo ${LINE_ICU_FILE} | ${CUT} -d'=' -f1`
                    VALUE=`echo ${LINE_ICU_FILE} | ${CUT} -d'=' -f2-`


                    FILE=`echo ${VALUE} | ${CUT} -d':' -f1`

                    # FILE et NAME_ICU_LIBRARY_FILE doivent avoir la meme valeur
                    DisplayTrace ""
                    DisplayTrace "    FILE           : \"${FILE}\"" 

                    # Recherche du fichier dans les repertoires
                    # FILE et NAME_ICU_LIBRARY_FILE doivent avoir la meme valeur
                    
                    SearchFileInDirectories "${NAME_ICU_LIBRARY_FILE}" "${SEARCH_DIRECTORIES}"
                    case $? in
                    0)
                        AddSettingsVariable "${NAME_ICU_PATH_IDENTIFIER}" "${DIR_FOUND}"
                        ;;

                    1)
                        RETURN_CODE=1
                        ;;

                    2)
                        # Fichier introuvable : cas inattendu 
                        RETURN_CODE=1
                        ;;

                    *)
                        DisplayTraceAndError "Invalid return value for SearchFileInDirectories ()" 
                        RETURN_CODE=1
                        ;;
                    esac   
                else
                    DisplayTraceAndError "Parameter \"file\" not found for \"${NAME_ICU_LIBRARY_FILE}\" library in \"${NAME_ICU_SECTION}\" section" 
                    RETURN_CODE=1
                fi
            fi
        else
            # Les parametres sont presents : rien a faire
            #     
            DisplayTrace "    Settings parameter \"${NAME_ICU_PATH_IDENTIFIER}\" found in installation settings file"
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# hpuModifyExecutableLibraryPath
###########################################################################

# Modification dans les executables des chemins de recherche des 
# bibliotheques dynamiques 
# 
#
# 2 arguments au moins.
#
#
# Cette fonction doit etre appelee apres l'installation des binaires HPU
# car elle examine et modifie les fichiers binaires HPU installes
#
#
# Cette fonction est specifique HPU.
#
### tester si elfdump, ... non trouves ###
### codes de retour
#

hpuModifyExecutableLibraryPath ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> hpuModifyExecutableLibraryPath"
    DisplayTrace ""
    DisplayTrace ""



    if [ $# -lt 2 ]
    then
        DisplayError "Missing arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        DIR_BINARIES="${LOCATION_PATH}/$1"
        shift 
        
        DisplayTrace "    DIR_BINARIES : \"${DIR_BINARIES}\" "


        LIB_SEARCH_PATH=""


        DisplayTrace "    Retrieval of parameter \"PATH_LIBICUDATA\" from installation settings file"

        PATH_LIBICUDATA=`GetSettingsVariable "PATH_LIBICUDATA"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PATH_LIBICUDATA}\" not found in ${FILE_INSTALL_SETTINGS}"
        fi
        DisplayTrace "    PATH_LIBICUDATA : \"${PATH_LIBICUDATA}\" "


        DisplayTrace "    Retrieval of parameter \"PATH_LIBICUUC\" from installation settings file"

        PATH_LIBICUUC=`GetSettingsVariable "PATH_LIBICUUC"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PATH_LIBICUUC}\" not found in ${FILE_INSTALL_SETTINGS}"
        fi
        DisplayTrace "    PATH_LIBICUUC : \"${PATH_LIBICUUC}\" "


        if [ "${PATH_LIBICUUC}" = "${PATH_LIBICUDATA}" ]
        then
            LIB_SEARCH_PATH="${PATH_LIBICUDATA}"
        else
            LIB_SEARCH_PATH="${PATH_LIBICUDATA}:${PATH_LIBICUUC}"
        fi
        DisplayTrace "    LIB_SEARCH_PATH : \"${LIB_SEARCH_PATH}\" "
        

        if [ ${RETURN_CODE} -eq 0 ]
        then

            DIR_CURRENT=`\pwd` 
            cd "${DIR_BINARIES}"

            for FILE_EXECUTABLE in $*
            do
                DisplayTrace ""
                DisplayTrace "    ------------------"
                DisplayTrace "    FILE_EXECUTABLE : \"${FILE_EXECUTABLE}\" "

                FILE_EXECUTABLE_MODIFIED="${FILE_EXECUTABLE}.modified"
                
                
                if [ -f "${FILE_EXECUTABLE}" ]
                then
                
                    case "${PLATFORM}" in

                    "linux"|"zlinux"|"plinux"|"plinuxle"|"ialinux"|"sun"|"sun_x86"|"hpia")
                        #

                        DisplayTrace ""
                        DisplayTrace "    Current RPATH value"

                        ActivateTraceRedirection

                        "${LOCATION_PATH}/install/modify_rpath" -v "${FILE_EXECUTABLE}" 2>&1
                        if [ $? -ne 0 ]
                        then
                            DisplayTraceAndError ""
                            DisplayTraceAndError "Error while displaying current RPATH value"
                        fi    
                        
                        DeactivateTraceRedirection

                        DisplayTrace ""
                        DisplayTrace "    Update of RPATH value with \"${LIB_SEARCH_PATH}\""

                        ActivateTraceRedirection

                        "${LOCATION_PATH}/install/modify_rpath" -v "${FILE_EXECUTABLE}" "${LIB_SEARCH_PATH}" 2>&1
                        if [ $? -ne 0 ]
                        then
                            DisplayTraceAndError ""
                            DisplayTraceAndError "Error while updating current RPATH value"
                            RETURN_CODE=1
                        fi    

                        DeactivateTraceRedirection

                        if [ "${RETURN_CODE}" -eq 0 ]
                        then
                            DisplayTrace ""
                            DisplayTrace "    New RPATH value"

                            ActivateTraceRedirection
                            
                            ${LOCATION_PATH}/install/modify_rpath -v "${FILE_EXECUTABLE}" 2>&1
                            if [ $? -ne 0 ]
                            then
                                DisplayTraceAndError ""
                                DisplayTraceAndError "Error while displaying new RPATH value"
                            fi    

                            DeactivateTraceRedirection
                        fi    

                        ;;


                    "aix")
                        #

                        DisplayTrace ""
                        DisplayTrace "    Current RPATH value"

                        ActivateTraceRedirection

                        \dump -X64 -H "${FILE_EXECUTABLE}" 2>&1
                        if [ $? -ne 0 ]
                        then
                            DisplayTraceAndError ""
                            DisplayTraceAndError "Error while displaying current RPATH value"
                        fi    
                        
                        # Obtention valeur courante
                        RPATH_CURRENT=`\dump -X64 -H "${FILE_EXECUTABLE}" \
                        | ${SED} -n '/^INDEX/,/^0/p' | \tail -n 1 \
                        | ${SED} -e 's/^0 *//' | ${SED} -e 's/ *$//g'`
                        # RPATH_NEW="${LIB_SEARCH_PATH}:${RPATH_CURRENT}"

                        DeactivateTraceRedirection
                        
                        DisplayTrace ""
                        DisplayTrace "    RPATH_CURRENT : \"${RPATH_CURRENT}\" "
                        # DisplayTrace "    RPATH_NEW     : \"${RPATH_NEW}\" "
                        
                        
                        # Determination si une des composantes de LIB_SEARCH_PATH 
                        # ne figure pas deja dans RPATH_CURRENT
                        
                        FILE_TEMP_RPATH_NEW="tmp_new_rpath_values.txt"
                        echo "${LIB_SEARCH_PATH}" | ${TR} ':' '\n' > "${DIR_TEMP}/${FILE_TEMP_RPATH_NEW}"
                        

                        # Astuce afin que des variables modifiees dans la 
                        # boucle le restent en sortie de cette boucle

                        exec 9<&0 < "${DIR_TEMP}/${FILE_TEMP_RPATH_NEW}"

                        RPATH_NEW=""
                        # Variable contnenant les options "-L" pour la commande
                        # ld : un repertoire a chaque fois 
                        OPTION_LD_AIX_DIRECTORY=""
                        
                        while read NEW_RPATH_ELEMENT
                        do
                            DisplayTrace ""
                            DisplayTrace "    NEW_RPATH_ELEMENT : \"${NEW_RPATH_ELEMENT}\" "
                            echo "${RPATH_CURRENT}" | ${TR} ':' '\n' | ${GREP} "${NEW_RPATH_ELEMENT}" > /dev/null 2>&1
                            if [ $? -eq 0 ]
                            then
                                DisplayTrace "    Directory \"${NEW_RPATH_ELEMENT}\" found in current RPATH value"
                            else
                                DisplayTrace "    Directory \"${NEW_RPATH_ELEMENT}\" not found in current RPATH value"
                                DisplayTrace "    It will be added to the RPATH value"
                                if [ "${RPATH_NEW}" == "" ]
                                then
                                    RPATH_NEW="${NEW_RPATH_ELEMENT}"
                                else
                                    RPATH_NEW="${RPATH_NEW}:${NEW_RPATH_ELEMENT}"
                                fi
                            fi
                            
                            # Il faut une directive -L pour chaque repertoire meme 
                            # s'il est deja present dans la valeur de RPATH courante
                            # sinon il y a la bibliotheque n'est pas trouvee lors 
                            # de l'edition de liens
                            OPTION_LD_AIX_DIRECTORY="${OPTION_LD_AIX_DIRECTORY} -L${NEW_RPATH_ELEMENT}"
                        done
                        exec 0<&9 9<&-
                        
                        ${RM} "${DIR_TEMP}/${FILE_TEMP_RPATH_NEW}"
                        
                        DisplayTrace "    RPATH_NEW               : \"${RPATH_NEW}\" "
                        DisplayTrace "    OPTION_LD_AIX_DIRECTORY : \"${OPTION_LD_AIX_DIRECTORY}\" "
                        
                        if [ "${RPATH_NEW}" == "" ]
                        then
                            DisplayTrace "    No new path to add to RPATH."
                        else
                            ### Pour option "-L" : un repertoire a chaque fois 

                            DisplayTrace ""
                            DisplayTrace "    Update of RPATH value by adding \"${RPATH_NEW}\""

                            ActivateTraceRedirection

                            \ld -b64 \
                                -o ${FILE_EXECUTABLE_MODIFIED} \
                                -blibpath:"${RPATH_NEW}:${RPATH_CURRENT}" \
                                "${FILE_EXECUTABLE}" \
                                ${OPTION_LD_AIX_DIRECTORY} \
                                -lc -lpthreads -licuuc55 -licudata55 -lodm -lcfg 2>&1
                            if [ $? -ne 0 ]
                            then
                                DisplayTraceAndError ""
                                DisplayTraceAndError "Error while updating current RPATH value"
                                RETURN_CODE=1
                            fi   

                            DeactivateTraceRedirection

                            if [ "${RETURN_CODE}" -eq 0 ]
                            then
                                DisplayTrace ""
                                DisplayTrace "    New RPATH value"

                                ActivateTraceRedirection

                                \dump -X64 -H "${FILE_EXECUTABLE_MODIFIED}" 2>&1
                                if [ $? -ne 0 ]
                                then
                                    DisplayTraceAndError ""
                                    DisplayTraceAndError "Error while displaying new RPATH value"
                                fi    

                                DeactivateTraceRedirection

                                ${MV} "${FILE_EXECUTABLE_MODIFIED}"  "${FILE_EXECUTABLE}" 
                                ### A Parametrer plus tard
                                ${CHMOD} 6755 "${FILE_EXECUTABLE}"
                            fi
                        fi
                        ;;


                    *)
                        #

                        DisplayTrace ""
                        DisplayTrace "    Unknown platform name : \"${PLATFORM}\" "
                        RETURN_CODE=1
                        ### Cas a finaliser ###
                        ;;

                    esac
            else
                # Fichier non trouve : pas d'erreur
                DisplayTrace "    File \"${FILE_EXECUTABLE}\" not found"
            fi
            done
            
            cd "${DIR_CURRENT}"
        fi

    fi

    return ${RETURN_CODE}
}


###########################################################################
# hpuDetermineDaemonBinaryName
###########################################################################

# Determination du nom du binaire du demon a lancer par inetd
# 
#
# Aucun argument.
#
#
# Cette fonction doit etre appelee apres l'installation des binaires HPU
# car elle examine les fichiers binaires HPU installes
#
# Cette fonction definit et initialise la variable HPU_DAEMON_NAME.
#
# Cette fonction est interne et specifique HPU.
#
#

hpuDetermineDaemonBinaryName ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> hpuDetermineDaemonBinaryName"
    DisplayTrace ""
    DisplayTrace ""

    HPU_DAEMON_NAME=""

    if [ -f "${LOCATION_PATH}/bin/db2hpudm" ]
    then
        HPU_DAEMON_NAME="db2hpudm"
    else
        DisplayError "No daemon binary found."
        DisplayTrace "No daemon binary found."
        RETURN_CODE=1
    fi
    
    DisplayTrace "    HPU_DAEMON_NAME : \"${HPU_DAEMON_NAME}\" "


    # Memorisation de la valeur dans fichier
    echo "HPU_DAEMON_NAME=${HPU_DAEMON_NAME}" >>  "${FILE_INSTALL_SETTINGS}"

    DisplayTrace "    HPU_DAEMON_NAME value written to installation settings file"

    return ${RETURN_CODE}
}


###########################################################################
# hpuMemorizeInstalledFeatures
###########################################################################

# Initialisation et memorisation dans le fichier de parametrage des variables
# INSTALL_FEATURE_CORE, INSTALL_FEATURE_NETWORK, INSTALL_FEATURE_PLUGIN
# 
#
# Aucun argument.
#
#
#
#
# Cette fonction est interne
#
#
#

hpuMemorizeInstalledFeatures ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> hpuMemorizeInstalledFeatures"
    
    DisplayTrace ""
    DisplayTrace ""


    # Initialisation de variables indiquant les features installes


    # Memorisation des variables pour chaque feature installe : valeur "1"
    
    for NAME in ${LIST_FEATURES_INSTALL}
    do
        NAME_TEMP=`ConvertToUpperCase "${NAME}"`
        NAME_IDENTIFIER="INSTALL_FEATURE_${NAME_TEMP}"
    
        DisplayTrace "    NAME_IDENTIFIER  : \"${NAME_IDENTIFIER}\" "
        
        # Memorisation de l'identificateur
        
        CheckSettingsVariable "${NAME_IDENTIFIER}"
        if [ $? -ne 0 ]
        then
            AddSettingsVariable "${NAME_IDENTIFIER}" "1"
        else
            DisplayTraceAndError "Information \"${NAME_IDENTIFIER}\" already found in settings file"
            RETURN_CODE=1
        fi

        # Declaration et initialisation de la variable dans le shell actuel
        \eval "${NAME_IDENTIFIER}=1"
    done
    
    
    # Memorisation des variables pour chaque feature non installe : valeur "0"
    
    for NAME in ${LIST_FEATURES_NO_INSTALL}
    do
        NAME_TEMP=`ConvertToUpperCase "${NAME}"`
        NAME_IDENTIFIER="INSTALL_FEATURE_${NAME_TEMP}"
    
        DisplayTrace "    NAME_IDENTIFIER  : \"${NAME_IDENTIFIER}\" "
        
        # Memorisation de l'identificateur
        
        CheckSettingsVariable "${NAME_IDENTIFIER}"
        if [ $? -ne 0 ]
        then
            AddSettingsVariable "${NAME_IDENTIFIER}" "0"
        else
            DisplayTraceAndError "Information \"${NAME_IDENTIFIER}\" already found in settings file"
            RETURN_CODE=1
        fi
        
        # Declaration et initialisation de la variable dans le shell actuel
        \eval "${NAME_IDENTIFIER}=0"
    done

    return ${RETURN_CODE}
}


###########################################################################
# hpuDetermineDB2InstallDirectory
###########################################################################

# Determination du repertoire d'installation de DB2 a partir du repertoire 
# d'instance (DEFAULT_HOME)
# 
#
#
#
#

hpuDetermineDB2InstallDirectory ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> hpuDetermineDB2InstallDirectory"
    
    DisplayTrace ""
    DisplayTrace ""



    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Recuperation repertoire d'installation de DB2 via la valeur du lien sqllib/lib

        if [ -h "${DEFAULT_HOME}/sqllib/lib" ]
        then
            # C'est un lien : recuperation du repertoire de DB2
            DB2_INSTALL_DIR=`${LS} -l "${DEFAULT_HOME}/sqllib/lib" | ${AWK} '{print $NF}'`
            DB2_INSTALL_DIR=`\dirname ${DB2_INSTALL_DIR}`

            DisplayTrace "    DB2 Directory: \"${DB2_INSTALL_DIR}\"  "

        else
            # Ce n'est pas un lien : cas atypique ou le repertoire 
            # ${DEFAULT_HOME}/sqllib/lib n'est pas un lien
            DisplayTrace "    Directory \"${DEFAULT_HOME}/sqllib/lib\" is not a symbolic link "

            # Prise en compte cas tres particuliers (non standards)

            # Test [ -x ] ?
            if [ -d "${DEFAULT_HOME}/sqllib/lib" ]
            then
                # Le repertoire lib est directement au niveau de l'instance
                DB2_INSTALL_DIR="${DEFAULT_HOME}/sqllib"
                DisplayTrace "    DB2 Directory: \"${DB2_INSTALL_DIR}\"  "

            else
                DisplayError "\"${DEFAULT_HOME}/sqllib/lib\" is not a directory."
                DisplayError "No suitable DB2 directory found"
                DisplayTrace "    No suitable DB2 directory found"
                DisplayError "Unable to find DB2 install directory"

                RETURN_CODE=1
            fi    
        fi

        if [ ${RETURN_CODE} -eq 0 ]
        then
            # Memorisation de la valeur dans fichier
            echo "DB2_INSTALL_DIR=${DB2_INSTALL_DIR}" >>  "${FILE_INSTALL_SETTINGS}"

            DisplayTrace "    DB2_INSTALL_DIR value written to installation settings file"
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# hpuRemoveInterimFixFiles
###########################################################################

#
# Suppression des fichiers des Interim Fix existants
#
# Aucun argument d'appel.
# 
# Specifique HPU.
#
# - Parcours des repertoires bin, msg/db2hpu avec suppression des fichiers 
#   XXX_HPUnnn*.
# - Parcours du repertoire ptf avec suppression des repertoires HPUnnn*
#
# Cette fonction sera appelee comme une commande avec l'action EXECUTER_COMMANDE.
#
#
# A faire : passer dans tous les repertoires d'une installation HPU 
#           y compris les sous-repertoires
#
#

hpuRemoveInterimFixFiles ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> hpuRemoveInterimFixFiles"


    if [ $# -ge 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    

        for SUBDIR_NAME in bin msg/db2hpu install/function tools
        do
            if [ -d "${LOCATION_PATH}/${SUBDIR_NAME}" ]
            then
                \cd ${LOCATION_PATH}/${SUBDIR_NAME}
                if [ $? -eq 0 ]
                then
                    DisplayTrace ""
                    DisplayTrace "    Subdirectory : ${SUBDIR_NAME}"
                    for NAME in `${LS} *_HPU[0123456789][0123456789][0123456789]* 2>/dev/null`
                    do
                        DisplayTrace "    Removal of file \"${NAME}\" "
                        ${RM} -f "${NAME}"
                    done
                fi
            fi                
        done


        if [ -d "${LOCATION_PATH}/ptf" ]
        then
            \cd ${LOCATION_PATH}/ptf
            if [ $? -eq 0 ]
            then
                DisplayTrace ""
                DisplayTrace "    Subdirectory : ptf"
                ${LS} -1 | while read NAME
                do
                    case "${NAME}" in 
                    HPU[0123456789][0123456789][0123456789]*)
                        DisplayTrace "    Removal subdirectory \"${NAME}\" "
                        ${RM} -rf "${NAME}"
                        ;;

                    *)
                        ;;
                    esac
                done    
            fi
        else
            DisplayTrace "    No directory \"${LOCATION_PATH}/ptf\" found."
        fi
            
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# hpuInitActionGlobalVariables 
###########################################################################

#
# Definition des variables qui seront exploitees dans des conditions pour 
# des actions (dans les fichiers .ini).
# 
#
# Aucun argument d'appel.
# 
# Specifique HPU.
#
# 
# 
# 
# 
# Attention a ne pas modifier involontairement des variables globales.
# 
#
#
# Variable initialisees :
#
#
#
# DB2_INSTALL_DIR : 
# Repertoire d'installation de DB2
#
#
# STANDALONE_INSTALLATION :
# 1 : DB2 n'est pas present sur la machine ("standalone")
# 0 : DB2 est present sur la machine (non "standalone")
# STANDALONE_INSTALLATION initialise a present dans InitVariables () de hpu_inst.sh
#
#
# INSTALL_FEATURE_CORE
# 1 : le feature "core" est installe
# 0 : le feature "core" n'est installe
#
#
# INSTALL_FEATURE_NETWORK
# 1 : le feature "network" est installe
# 0 : le feature "network" n'est installe
#
#
# INSTALL_FEATURE_PLUGIN
# 1 : le feature "plugin" est installe
# 0 : le feature "plugin" n'est installe
#
#
#
#
# La plupart de ces variables sont sauvegardees dans un fichiers afin d'etre
# utilisees directement lors de la desinstallation
# 
#
#

hpuInitActionGlobalVariables ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> hpuInitActionGlobalVariables"
    
    DisplayTrace ""
    DisplayTrace "    Initialisation of global variables for actions"


    HPU_BIT_WIDTH=0


    case "${CONTEXT}" in

    "inst")    

        # Modification/determination de la valeur de STANDALONE_INSTALLATION, USE_EXISTING_SETTINGS
        # STANDALONE_INSTALLATION initialisee/recuperee a present dans InitVariables () de hpu_inst.sh
        # (mais non encore memorisee)

        case "${SUB_CONTEXT}" in

        "FIXPACK")
            # Recuperation de l'information STANDALONE_INSTALLATION
            # STANDALONE_INSTALLATION initialise/recupere a present dans InitVariables () de hpu_inst.sh


            if [ "${STANDALONE_INSTALLATION}" = "0" ]
            then
                # Tout ce qui suit n'a de sens que si DB2 est present

                # En desinstallation la variable DEFAULT_HOME n'est pas initialisee
                # Determination du repertoire d'installation de DB2 du lien qui a ete cree dans 
                # lib

                # Recuperation de l'information DB2_INSTALLATION_INFO

                if [ ${RETURN_CODE} -eq 0 ]
                then
                    DisplayTrace "    Retrieval of parameter \"DB2_INSTALLATION_INFO}\" from installation settings file"

                    DB2_INSTALLATION_INFO=`${GREP} "DB2_INSTALLATION_INFO" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                    if [ $? -ne 0 ]
                    then
                        # Information non memorisee
                        DisplayError "Parameter \"DB2_INSTALLATION_INFO}\" not found in ${FILE_INSTALL_SETTINGS}"
                        RETURN_CODE=1
                    else
                        DisplayTrace "    DB2_INSTALLATION_INFO : \"${DB2_INSTALLATION_INFO}\" "
                    fi
                fi    
            fi    

            ;;


        "")
            # Memorisation de STANDALONE_INSTALLATION
            echo "STANDALONE_INSTALLATION=${STANDALONE_INSTALLATION}" >> "${FILE_INSTALL_SETTINGS}"

            DisplayTrace "    STANDALONE_INSTALLATION value written to installation settings file"
            
            ;;


        *)
            DisplayTrace "Unknown sub-context \"${SUB_CONTEXT}\" " 
            DisplayError "Unknown sub-context \"${SUB_CONTEXT}\" "
            RETURN_CODE=2            
            ;;

        esac

        DisplayTrace "    STANDALONE_INSTALLATION : \"${STANDALONE_INSTALLATION}\" "



        if [ "${STANDALONE_INSTALLATION}" = "0" ]
        then
            # Tout ce qui suit n'a de sens que si DB2 est present

            case "${SUB_CONTEXT}" in

            "FIXPACK")
                # Recuperation de l'information DB2_INSTALLATION_INFO

                if [ ${RETURN_CODE} -eq 0 ]
                then
                    DisplayTrace "    Retrieval of parameter \"DB2_INSTALLATION_INFO}\" from installation settings file"

                    DB2_INSTALLATION_INFO=`${GREP} "DB2_INSTALLATION_INFO" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                    if [ $? -ne 0 ]
                    then
                        # Information non memorisee
                        DisplayError "Parameter \"DB2_INSTALLATION_INFO}\" not found in ${FILE_INSTALL_SETTINGS}" 
                        RETURN_CODE=1
                    else
                        DisplayTrace "    DB2_INSTALLATION_INFO : \"${DB2_INSTALLATION_INFO}\" "
                    fi
                fi    
                
                ;;

            "")

                if [ "${RETURN_CODE}" -eq 0 ]
                then
                    # Determination du repertoire d'installation de DB2 a l'aide de l'outil specifique db2dirs

                    # Entre les apostrophes : caractere tabulation
                    CHAR_TABULATION='	'

                    # Commande "head" a mettre en variable shell

                    DisplayTrace "    Output of db2dirs command:"
                    DisplayTrace "    --------------------------------------------"
                    ${LOCATION_PATH}/install/db2dirs  
                    DisplayTrace "    --------------------------------------------"
                    DisplayTrace ""


                    # Astuce afin que DB2_INSTALLATION_INFO reste modifie en sortie de la 
                    # boucle "while"
                    ${LOCATION_PATH}/install/db2dirs > "${LOCATION_PATH}/install/temp_db2dirs"
                    exec 9<&0 < "${LOCATION_PATH}/install/temp_db2dirs"

                    DB2_INSTALLATION_INFO=""

                    # ${LOCATION_PATH}/install/db2dirs | while read LINE
                    while read LINE
                    do

                        DisplayTrace ""
                        DisplayTrace "    LINE : \"${LINE}\"" 

                        DB2_INSTALL_DIR=`echo "${LINE}" | ${CUT} -d"${CHAR_TABULATION}" -f 1`
                        DB2_VERSION=`echo "${LINE}" | ${CUT} -d"${CHAR_TABULATION}" -f 2`

                        DisplayTrace "    DB2_INSTALL_DIR : \"${DB2_INSTALL_DIR}\" "
                        DisplayTrace "    DB2_VERSION     : \"${DB2_VERSION}\" "

                        # Controle existence 
                        if [ ! -d "${DB2_INSTALL_DIR}" ] 
                        then
                            DisplayTrace "    DB2 install directory \"${DB2_INSTALL_DIR}\" not found"
                            DisplayTrace "    Skipping directory..."
                            continue 
                        fi


                        # Memorisation du repertoire et de la version

                        if [ "${DB2_INSTALLATION_INFO}" = "" ]
                        then
                            DB2_INSTALLATION_INFO="${DB2_INSTALL_DIR}:${DB2_VERSION}"
                        else    
                            DB2_INSTALLATION_INFO="${DB2_INSTALLATION_INFO}|${DB2_INSTALL_DIR}:${DB2_VERSION}"
                        fi
                        DisplayTrace "    DB2_INSTALLATION_INFO : \"${DB2_INSTALLATION_INFO}\" "

                    done

                    exec 0<&9 9<&-


                    if [ ${RETURN_CODE} -eq 0 ]
                    then
                        # Memorisation de DB2_INSTALLATION_INFO
                        echo "DB2_INSTALLATION_INFO=${DB2_INSTALLATION_INFO}" >> "${FILE_INSTALL_SETTINGS}"

                        DisplayTrace "    DB2_INSTALLATION_INFO value written to installation settings file"
                    fi

                    ${RM} ${LOCATION_PATH}/install/temp_db2dirs

                fi
                ;;

            *)
                DisplayTrace "Unknown sub-context \"${SUB_CONTEXT}\" " 
                DisplayError "Unknown sub-context \"${SUB_CONTEXT}\" "
                RETURN_CODE=2            
                ;;

            esac

        fi    



        case "${SUB_CONTEXT}" in

        "FIXPACK")
           
            # Recuperation des informations INSTALL_FEATURE_CORE, 
            # INSTALL_FEATURE_NETWORK, INSTALL_FEATURE_PLUGIN
            # Il se peut qu'elles n'aient pas ete memorisees (HPU 3.1 Full, WGE, 
            # T&B n'incluaient pas encore ces informations).
            # Dans ce cas on les cree.
            #
            # Peut-etre effets inattendus a verifier/prevoir.
        
            # Test presence des informations base uniquement sur une seule variable au lieu des trois
             
            ${GREP} "INSTALL_FEATURE_CORE" "${FILE_INSTALL_SETTINGS}" >/dev/null 2>&1
            if [ $? -eq 0 ]
            then
        
                # Recuperation de l'information INSTALL_FEATURE_CORE

                DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_CORE\" from installation settings file"

                INSTALL_FEATURE_CORE=`${GREP} "INSTALL_FEATURE_CORE" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                if [ $? -ne 0 ]
                then
                    # Information non memorisee
                    DisplayError "Parameter \"INSTALL_FEATURE_CORE}\" not found in ${FILE_INSTALL_SETTINGS}"
                    RETURN_CODE=1
                fi
                DisplayTrace "    INSTALL_FEATURE_CORE  : \"${INSTALL_FEATURE_CORE}\" "


                # Recuperation de l'information INSTALL_FEATURE_NETWORK

                DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_NETWORK\" from installation settings file"

                INSTALL_FEATURE_NETWORK=`${GREP} "INSTALL_FEATURE_NETWORK" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                if [ $? -ne 0 ]
                then
                    # Information non memorisee
                    DisplayError "Parameter \"INSTALL_FEATURE_NETWORK}\" not found in ${FILE_INSTALL_SETTINGS}" 
                    RETURN_CODE=1
                fi
                DisplayTrace "    INSTALL_FEATURE_NETWORK : \"${INSTALL_FEATURE_NETWORK}\" "

            else
            
                # Initialisation de variables indiquant les features installes
                hpuMemorizeInstalledFeatures            

                RETURN_CODE=$?
            
            fi            
            ;;


        "")
            # Initialisation de variables indiquant les features installes
            hpuMemorizeInstalledFeatures            

            RETURN_CODE=$?
            ;;

        *)
            DisplayTrace "Unknown sub-context \"${SUB_CONTEXT}\" " 
            DisplayError "Unknown sub-context \"${SUB_CONTEXT}\" "
            RETURN_CODE=2            
            ;;

        esac
        ;;


    "uninst")

        # Initialisation de la valeur de STANDALONE_INSTALLATION
        
        ### # Determination si DB2 est present ou non par l'existence des liens lib/db2 et
        ### # lib/db264
        ### 
        ### if [ ! -h "${LOCATION_PATH}/lib/db2" -a ! -h "${LOCATION_PATH}/lib/db264" ]
        
        
        
        ### # Recuperation de l'information STANDALONE_INSTALLATION
        ### 
        ### DisplayTrace "Retrieval of parameter \"STANDALONE_INSTALLATION}\" from installation settings file"
        ### 
        ### STANDALONE_INSTALLATION=`${GREP} "STANDALONE_INSTALLATION" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
        ### if [ $? -ne 0 ]
        ### then
        ###     # Information non memorisee
        ###     echo "Parameter \"STANDALONE_INSTALLATION\" not found in ${FILE_INSTALL_SETTINGS}"  1>&2
        ###     RETURN_CODE=1
        ### else
        ###     DisplayTrace "STANDALONE_INSTALLATION : \"${STANDALONE_INSTALLATION}\" "
        ### fi
        


        if [ "${STANDALONE_INSTALLATION}" = "0" ]
        then
            # Tout ce qui suit n'a de sens que si DB2 est present

            # En desinstallation la variable DEFAULT_HOME n'est pas initialisee
            # Determination du repertoire d'installation de DB2 du lien qui a ete cree dans 
            # lib
            
            # Recuperation de l'information DB2_INSTALLATION_INFO

            if [ ${RETURN_CODE} -eq 0 ]
            then
                DisplayTrace "    Retrieval of parameter \"DB2_INSTALLATION_INFO}\" from installation settings file"

                DB2_INSTALLATION_INFO=`${GREP} "DB2_INSTALLATION_INFO" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                if [ $? -ne 0 ]
                then
                    # Information non memorisee
                    DisplayError "Parameter \"DB2_INSTALLATION_INFO}\" not found in ${FILE_INSTALL_SETTINGS}"
                    RETURN_CODE=1
                else
                    DisplayTrace "    DB2_INSTALLATION_INFO : \"${DB2_INSTALLATION_INFO}\" "
                fi
            fi    
        fi    
        
        
        # Recuperation de l'information INSTALL_FEATURE_CORE

        DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_CORE\" from installation settings file"

        INSTALL_FEATURE_CORE=`${GREP} "INSTALL_FEATURE_CORE" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayError "Parameter \"INSTALL_FEATURE_CORE}\" not found in ${FILE_INSTALL_SETTINGS}"
            RETURN_CODE=1
        fi
        DisplayTrace "    INSTALL_FEATURE_CORE  : \"${INSTALL_FEATURE_CORE}\" "


        # Recuperation de l'information INSTALL_FEATURE_NETWORK

        DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_NETWORK\" from installation settings file"

        INSTALL_FEATURE_NETWORK=`${GREP} "INSTALL_FEATURE_NETWORK" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayError "Parameter \"INSTALL_FEATURE_NETWORK}\" not found in ${FILE_INSTALL_SETTINGS}" 
            RETURN_CODE=1
        fi
        DisplayTrace "    INSTALL_FEATURE_NETWORK : \"${INSTALL_FEATURE_NETWORK}\" "       
        

        # Recuperation de l'information HPU_SERVICE_NAME

        DisplayTrace "    Retrieval of parameter \"HPU_SERVICE_NAME\" from installation settings file"

        HPU_SERVICE_NAME=`${GREP} "HPU_SERVICE_NAME" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayError "Parameter \"HPU_SERVICE_NAME}\" not found in ${FILE_INSTALL_SETTINGS}" 
            RETURN_CODE=1
        fi
        DisplayTrace "    HPU_SERVICE_NAME  : \"${HPU_SERVICE_NAME}\" "

        
        # Recuperation de l'information HPU_SERVICE_PORT

        DisplayTrace "    Retrieval of parameter \"HPU_SERVICE_PORT\" from installation settings file"

        HPU_SERVICE_PORT=`${GREP} "HPU_SERVICE_PORT" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayError "Parameter \"HPU_SERVICE_PORT}\" not found in ${FILE_INSTALL_SETTINGS}" 
            RETURN_CODE=1
        fi
        DisplayTrace "    HPU_SERVICE_PORT  : \"${HPU_SERVICE_PORT}\" "

        ;;


    *)
        DisplayError "Unknown context \"${CONTEXT}\" "
        RETURN_CODE=2    
        ;;
    esac
    
    return ${RETURN_CODE}
}

